<!DOCTYPE html>
<html>
<head>
  <meta http-equiv="content-type" content="text/html;charset=utf-8">
  <title>{{ title }}</title>
  <link rel="stylesheet" href="{{ root_path }}/pyccoon.css">
  {{#mathjax?}}
  <script type="text/x-mathjax-config">
  MathJax.Hub.Config({
    tex2jax: {inlineMath: [['$','$'], ['\\(','\\)']]}
  });
  </script>
  <script type="text/javascript" src="http://cdn.mathjax.org/mathjax/latest/MathJax.js?config=TeX-AMS-MML_HTMLorMML"></script>
  {{/mathjax?}}
</head>
<body>
<div id='container'>
  <a id="top"></a>
  <div id="background"></div>
  <div id="footer">
    <a class="btn" id="project-name" href="{{ root_path }}/index.html">{{{ project_name }}}</a>
    <a class="btn" class="source source-1" href="#top">Back to top</a>
    {{#contents?}}
    <div id="jump_to" class="btn">
    Contents
    <div id="jump_wrapper">
      <div id="jump_page">
        {{#contents}}
          <a class="source source-{{ level }}" href="{{ url }}">{{& basename }}</a>
        {{/contents}}
      </div>
    </div>
    </div>
    {{/contents?}}
    <div class="generation-time">
        Generated at
        <a target="_blank" href="http://ckald.github.io/pyccoon/" title="Pyccoon: side-to-side documentation generator">
            <img id="pyccoon-icon" src="{{ root_path }}/pyccoon_icon.svg" alt="Pyccoon: side-to-side documentation generator" /></a>
        <code>{{ generation_time }}</code>/
        by a diligent
        <a target="_blank" href="http://ckald.github.io/pyccoon/" title="Pyccoon: side-to-side documentation generator">pyccoon</a>
    </div>
  </div>
  <div class='section'>
    <div class='docs'>
      <h1 id="breadcrumbs">{{#breadcrumbs}}/ <a href="{{ path }}">{{ title }}</a> {{/breadcrumbs}}</h1>
      <ul id="children">{{#children}}<li><a href="{{ path }}"{{#isdir}} class="directory"{{/isdir}}>{{ title }}</a></li>{{/children}}</ul>
    </div>
  </div>
  <div class='clearall'>
  {{#sections}}
  <div class='section'>
    <a id="section-{{ num }}" class="section-anchor"></a>
    <section>
        <div class='docs'>
          <div class='octowrap'>
            <a class='octothorpe' href='#section-{{ num }}'>#</a>
          </div>
          {{{ docs_html }}}
        </div>
        <div class='code'>
          {{{ code_html }}}
        </div>
        <div class='clearall'></div>
    </section>
  </div>
  {{/sections}}
</div>
</body>
